@@ -1,17 +1,6 @@
-from __future__ import absolute_import, division, print_function
-
-import sys
 import base64
 import uuid
-
-if sys.version_info >= (3, 0):
-    unicode = str
-    from io import StringIO, BytesIO
-else:
-    from StringIO import StringIO
-    BytesIO = StringIO
-
-
+from io import StringIO, BytesIO
 import umsgpack
 import numpy as np
 
@@ -20,7 +9,7 @@ from . import transformations as tf
 
 class SceneElement(object):
     def __init__(self):
-        self.uuid = unicode(uuid.uuid1())
+        self.uuid = str(uuid.uuid1())
 
 
 class ReferenceSceneElement(SceneElement):
@@ -202,7 +191,7 @@ class PngImage(Image):
     def lower(self, object_data):
         return {
             u"uuid": self.uuid,
-            u"url": unicode("data:image/png;base64," + base64.b64encode(self.data).decode('ascii'))
+            u"url": str("data:image/png;base64," + base64.b64encode(self.data).decode('ascii'))
         }
 
 
@@ -280,7 +269,7 @@ class OrthographicCamera(SceneElement):
         self.near = near
         self.far = far
         self.zoom = zoom
-    
+
     def lower(self):
         data = {
             u"object": {
@@ -332,15 +321,12 @@ def pack_numpy_array(x):
 
 
 def data_from_stream(stream):
-    if sys.version_info >= (3, 0):
-        if isinstance(stream, BytesIO):
-            data = stream.read().decode(encoding='utf-8')
-        elif isinstance(stream, StringIO):
-            data = stream.read()
-        else:
-            raise ValueError('Stream must be instance of StringIO or BytesIO, not {}'.format(type(stream)))
-    else:
+    if isinstance(stream, BytesIO):
+        data = stream.read().decode(encoding='utf-8')
+    elif isinstance(stream, StringIO):
         data = stream.read()
+    else:
+        raise ValueError('Stream must be instance of StringIO or BytesIO, not {}'.format(type(stream)))
     return data
 
 
@@ -401,15 +387,12 @@ class StlMeshGeometry(MeshGeometry):
 
     @staticmethod
     def from_stream(f):
-        if sys.version_info >= (3, 0):
-            if isinstance(f, BytesIO):
-                arr  = np.frombuffer(f.read(), dtype=np.uint8)
-            elif isinstance(f, StringIO):
-                arr = np.frombuffer(bytes(f.read(), "utf-8"), dtype=np.uint8)
-            else:
-                raise ValueError('Stream must be instance of StringIO or BytesIO, not {}'.format(type(f)))
-        else:
+        if isinstance(f, BytesIO):
             arr  = np.frombuffer(f.read(), dtype=np.uint8)
+        elif isinstance(f, StringIO):
+            arr = np.frombuffer(bytes(f.read(), "utf-8"), dtype=np.uint8)
+        else:
+            raise ValueError('Stream must be instance of StringIO or BytesIO, not {}'.format(type(f)))
         _, extcode = threejs_type(np.uint8)
         encoded = umsgpack.Ext(extcode, arr.tobytes())
         return MeshGeometry(encoded, u"stl")
@@ -417,9 +400,13 @@ class StlMeshGeometry(MeshGeometry):
 
 class TriangularMeshGeometry(Geometry):
     """
-    A mesh consisting of an arbitrary collection of triangular faces. To construct one, you need to pass in a collection of vertices as an Nx3 array and a collection of faces as an Mx3 array. Each element of `faces` should be a collection of 3 indices into the `vertices` array.
+    A mesh consisting of an arbitrary collection of triangular faces. To
+    construct one, you need to pass in a collection of vertices as an Nx3 array
+    and a collection of faces as an Mx3 array. Each element of `faces` should
+    be a collection of 3 indices into the `vertices` array.
 
-    For example, to create a square made out of two adjacent triangles, we could do:
+    For example, to create a square made out of two adjacent triangles, we
+    could do:
 
     vertices = np.array([
         [0, 0, 0],  # the first vertex is at [0, 0, 0]
@@ -433,9 +420,14 @@ class TriangularMeshGeometry(Geometry):
     ])
 
     mesh = TriangularMeshGeometry(vertices, faces)
+
+    To set the color of the mesh by vertex, pass an Nx3 array containing the
+    RGB values (in range [0,1]) of the vertices to the optional `color`
+    argument, and set `vertexColors=True` in the Material.
     """
     __slots__ = ["vertices", "faces"]
-    def __init__(self, vertices, faces):
+
+    def __init__(self, vertices, faces, color=None):
         super(TriangularMeshGeometry, self).__init__()
 
         vertices = np.asarray(vertices, dtype=np.float32)
@@ -444,15 +436,20 @@ class TriangularMeshGeometry(Geometry):
         assert faces.shape[1] == 3, "`faces` must be an Mx3 array"
         self.vertices = vertices
         self.faces = faces
+        if color is not None:
+            color = np.asarray(color, dtype=np.float32)
+            assert np.array_equal(vertices.shape, color.shape), "`color` must be the same shape as vertices"
+        self.color = color
 
     def lower(self, object_data):
+        attrs = {u"position": pack_numpy_array(self.vertices.T)}
+        if self.color is not None:
+            attrs[u"color"] = pack_numpy_array(self.color.T)
         return {
             u"uuid": self.uuid,
             u"type": u"BufferGeometry",
             u"data": {
-                u"attributes": {
-                    u"position": pack_numpy_array(self.vertices.T)
-                },
+                u"attributes": attrs,
                 u"index": pack_numpy_array(self.faces.T)
             }
         }
