@@ -175,9 +175,7 @@ class ExtensibleObjectLoader extends THREE.ObjectLoader {
                 if (json.mtl_library) {
                     let mtl_loader = new MTLLoader(manager);
                     let mtl_parse_result = mtl_loader.parse(json.mtl_library + "\n", "");
-                    console.log(mtl_parse_result);
                     let materials = MtlObjBridge.addMaterialsFromMtlLoader(mtl_parse_result);
-                    console.log(materials);
                     loader.addMaterials(materials);
                     this.onTextureLoad();
                 }
@@ -318,10 +316,13 @@ class SceneNode {
                 let cast_shadow_controller = this.folder.add(this.object, "castShadow");
                 cast_shadow_controller.onChange(() => this.on_update());
                 this.controllers.push(cast_shadow_controller);
-                // Light source radius
-                let radius_controller = this.folder.add(this.object.shadow, "radius").min(0).step(0.05).max(3.0);
-                radius_controller.onChange(() => this.on_update());
-                this.controllers.push(radius_controller);
+
+                if (this.object.shadow !== undefined) {
+                    // Light source radius
+                    let radius_controller = this.folder.add(this.object.shadow, "radius").min(0).step(0.05).max(3.0);
+                    radius_controller.onChange(() => this.on_update());
+                    this.controllers.push(radius_controller);
+                }
             }
             // Point light falloff distance
             if (this.object.distance !== undefined){
@@ -347,6 +348,27 @@ class SceneNode {
             this.object.quaternion.set(value[0], value[1], value[2], value[3]);
         } else if (property === "scale") {
             this.object.scale.set(value[0], value[1], value[2]);
+        } else if (property === "color") {
+            function setNodeColor(node, value) {
+                if (node.material) {
+                    node.material.color.setRGB(value[0], value[1], value[2])
+
+                    let alpha = value[3]
+                    node.material.opacity = alpha 
+                    if(alpha != 1.) {
+                       node.material.transparent = true
+                    } 
+                    else {
+                        node.material.transparent = false
+                    }
+                }
+                for (let child of node.children) {
+                    setNodeColor(child, value);
+                }
+            }
+            setNodeColor(this.object, value)
+        } else if (property == "top_color" || property == "bottom_color") {
+            this.object[property] = value.map((x) => x * 255);
         } else {
             this.object[property] = value;
         }
@@ -481,7 +503,7 @@ class Animator {
     }
 
     setup_capturer(format) {
-        this.capturer = new CCapture({
+        this.capturer = new window.CCapture({
             format: format,
             name: "meshcat_" + String(Date.now())
         });
@@ -607,7 +629,8 @@ class Animator {
         this.progress = 0;
         for (let animation of animations) {
             let target = this.viewer.scene_tree.find(animation.path).object;
-            let clip = this.loader.parseAnimations([animation.clip])[0];
+            let animations = this.loader.parseAnimations([animation.clip]);
+            let clip = Object.values(animations)[0];
             let action = this.mixer.clipAction(clip, target);
             action.clampWhenFinished = options.clampWhenFinished;
             action.setLoop(options.loopMode, options.repetitions);
@@ -689,21 +712,26 @@ function gradient_texture(top_color, bottom_color) {
 
 
 class Viewer {
-    constructor(dom_element, animate) {
+    constructor(dom_element, animate, renderer) {
         this.dom_element = dom_element;
-        this.renderer = new THREE.WebGLRenderer({antialias: true, alpha: true});
-        this.renderer.shadowMap.enabled = true;
-        this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
-        this.dom_element.appendChild(this.renderer.domElement);
+        if (renderer === undefined) {
+            this.renderer = new THREE.WebGLRenderer({antialias: true, alpha: true});
+            this.renderer.shadowMap.enabled = true;
+            this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
+            this.dom_element.appendChild(this.renderer.domElement);
+        } else {
+            this.renderer = renderer;
+        }
 
         this.scene = create_default_scene();
+        this.gui_controllers = {};
         this.create_scene_tree();
 
         this.add_default_scene_elements();
         this.set_dirty();
 
         this.create_camera();
-
+        this.num_messages_received = 0;
 
         // TODO: probably shouldn't be directly accessing window?
         window.onload = (evt) => this.set_3d_pane_size();
@@ -828,9 +856,9 @@ class Viewer {
         this.gui.close();
 
         this.set_property(["Background"],
-            "top_color", [135, 206, 250]); // lightskyblue
+            "top_color", [135/255, 206/255, 250/255]); // lightskyblue
         this.set_property(["Background"],
-            "bottom_color", [25, 25, 112]); // midnightblue
+            "bottom_color", [25/255, 25/255, 112/255]); // midnightblue
         this.scene_tree.find(["Background"]).on_update = () => {
             if (this.scene_tree.find(["Background"]).object.visible)
                 this.show_background();
@@ -955,13 +983,32 @@ class Viewer {
 
     set_control(name, callback, value, min, max, step) {
         let handler = {};
+        if (name in this.gui_controllers) {
+            this.gui.remove(this.gui_controllers[name]);
+        }
         if (value !== undefined) {
             handler[name] = value;
-            let controller = this.gui.add(handler, name, min, max, step);
-            controller.onChange(eval(callback));
+            this.gui_controllers[name] = this.gui.add(
+                handler, name, min, max, step);
+            this.gui_controllers[name].onChange(eval(callback));
         } else {
             handler[name] = eval(callback);
-            this.gui.add(handler, name);
+            this.gui_controllers[name] = this.gui.add(handler, name);
+        }
+    }
+
+    set_control_value(name, value) {
+        if (name in this.gui_controllers && this.gui_controllers[name] 
+            instanceof dat.controllers.NumberController) {
+            this.gui_controllers[name].setValue(value);
+            this.gui_controllers[name].updateDisplay();
+        }
+    }
+
+    delete_control(name) {
+        if (name in this.gui_controllers) {
+            this.gui.remove(this.gui_controllers[name]);
+            delete this.gui_controllers[name];
         }
     }
 
@@ -985,6 +1032,18 @@ class Viewer {
             this.set_animation(cmd.animations, cmd.options);
         } else if (cmd.type == "set_control") {
             this.set_control(cmd.name, cmd.callback, cmd.value, cmd.min, cmd.max, cmd.step);
+        } else if (cmd.type == "set_control_value") {
+            this.set_control_value(cmd.name, cmd.value);
+        } else if (cmd.type == "delete_control") {
+            this.delete_control(cmd.name);
+        } else if (cmd.type == "capture_image") {
+            let imgdata = this.capture_image();
+            this.connection.send(JSON.stringify({
+                'type': 'img',
+                'data': imgdata
+            }));
+        } else if (cmd.type == "save_image") {
+            this.save_image()
         }
         this.set_dirty();
     }
@@ -993,8 +1052,9 @@ class Viewer {
         let decoded = msgpack.decode(bytearray);
         this.handle_command(decoded);
     }
-
+    
     handle_command_message(message) {
+        this.num_messages_received++;
         this.handle_command_bytearray(new Uint8Array(message.data));
     }
 
@@ -1005,9 +1065,7 @@ class Viewer {
         if (location.protocol == "https:") {
             url = url.replace("ws:", "wss:");
         }
-        console.log(url);
         this.connection = new WebSocket(url);
-        console.log("connection:", this.connection);
         this.connection.binaryType = "arraybuffer";
         this.connection.onmessage = (msg) => this.handle_command_message(msg);
         this.connection.onclose = function(evt) {
